'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _stdEnv = require('std-env');

var _stdEnv2 = _interopRequireDefault(_stdEnv);

var _cli = require('./utils/cli');

var _reporters = require('./reporters');

var _profile = require('./profile');

var _profile2 = _interopRequireDefault(_profile);

var _request = require('./utils/request');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Default plugin options
const DEFAULTS = {
  name: 'webpack',
  color: 'green',
  profile: false,
  stream: process.stdout,
  reporters: [],
  reporter: null,
  log: !!_stdEnv2.default.minimalCLI,
  bars: !_stdEnv2.default.minimalCLI
};

// Mapping from name => { isRunning, details, progress, msg, request }
const globalStates = {};

class WebpackBarPlugin extends _webpack2.default.ProgressPlugin {
  constructor(options) {
    super();

    this.options = Object.assign({}, DEFAULTS, options);

    // this.handler will be called by webpack.ProgressPlugin
    this.handler = (percent, msg, ...details) => this.updateProgress(percent, msg, details);

    // Keep our state in shared ojbect
    this.states = globalStates;
    if (!this.states[this.options.name]) {
      this.states[this.options.name] = {
        isRunning: false,
        color: this.options.color,
        profile: this.options.profile ? new _profile2.default(this.options.name) : null
      };
    }
    this.state = this.states[this.options.name];

    // Reporters
    this.reporters = Array.from(this.options.reporters || []);

    if (this.options.reporter) {
      this.reporters.unshift(this.options.reporter);
    }

    if (this.options.bars) {
      this.reporters.unshift(new _reporters.BarsReporter());
    }

    if (this.options.profile) {
      this.reporters.unshift(new _reporters.ProfileReporter());
    }

    if (this.options.log) {
      this.reporters.unshift(new _reporters.LogReporter());
    }
  }

  callReporters(fn, payload = {}) {
    for (const reporter of this.reporters) {
      if (typeof reporter[fn] === 'function') {
        try {
          reporter[fn](this, payload);
        } catch (e) {
          _cli.consola.error(e);
        }
      }
    }
  }

  hasRunning() {
    return Object.values(this.states).some(state => state.isRunning);
  }

  hasErrors() {
    return Object.values(this.states).some(state => state.stats && state.stats.hasErrors());
  }

  apply(compiler) {
    super.apply(compiler);

    const hook = stats => {
      this.state.stats = stats;
      try {
        if (!this.hasRunning()) {
          this.callReporters('done');
        }
      } catch (e) {
        _cli.consola.error(e);
      }
    };

    if (compiler.hooks) {
      compiler.hooks.done.tap('WebpackBar', hook);
    } else {
      compiler.plugin('done', hook);
    }
  }

  updateProgress(percent, msg, details = []) {
    const progress = Math.floor(percent * 100);
    const isRunning = progress < 100;

    const wasRunning = this.state.isRunning;

    Object.assign(this.state, {
      details,
      progress,
      msg: isRunning && msg ? msg : '',
      request: (0, _request.parseRequest)(details[2]),
      elapsed: process.hrtime(this.state.start),
      isRunning
    });

    if (!wasRunning && isRunning) {
      // Started
      this.state.start = process.hrtime();
      this.callReporters('compiling');
    } else if (wasRunning && !isRunning) {
      // Finished
      this.callReporters('compiled');
    }

    if (this.options.profile) {
      this.state.profile.onRequest(this.state.request);
    }

    this.callReporters('update');
  }
}
exports.default = WebpackBarPlugin;